@@ -4,7 +4,7 @@
     begin                : Mon Feb 12 23:45:41 EST 2001
     copyright            : (C) 2001-2003 by Sarang Lakare
     email                : sarang#users.sourceforge.net
-    copyright            : (C) 2003-2004 by Olaf Schmidt
+    copyright            : (C) 2003-2005 by Olaf Schmidt
     email                : ojschmidt@kde.org
  ***************************************************************************/
 
@@ -24,6 +24,8 @@
 
 // include files for Qt
 #include <qbitmap.h>
+#include <qpixmap.h>
+#include <qimage.h>
 #include <qcursor.h>
 #include <qglobal.h>
 #include <qpainter.h>
@@ -36,6 +38,9 @@
 #include <kdebug.h>
 #include <klocale.h>
 
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>
+
 // include bitmaps for cursors
 static uchar left_ptr_bits[] = {
    0x00, 0x00, 0x08, 0x00, 0x18, 0x00, 0x38, 0x00, 0x78, 0x00, 0xf8, 0x00,
@@ -73,6 +78,48 @@ static uchar phand_bits[] = {
   0x00, 0x00, 0x00, 0x00,  0x00, 0x00, 0x00, 0x00 };
 
 
+
+static bool obscuredRegion (QRegion &region, Window winId, Window ignoreId, Window start = 0, int level = -1) {
+  Window root, parent, *children; uint nchildren;
+  if (0 == start)
+    start = qt_xrootwin();
+
+  bool winIdFound = false;
+  if (0 != XQueryTree (qt_xdisplay(), start, &root, &parent, &children, &nchildren)) {
+    for (uint i=0; i < nchildren; ++i) {
+      if (winIdFound) {
+        if (ignoreId != children [i]) {
+          XWindowAttributes atts;
+          XGetWindowAttributes (qt_xdisplay(), children [i], &atts);
+          if (atts.map_state == IsViewable)
+            region -= QRegion (atts.x, atts.y, atts.width, atts.height, QRegion::Rectangle);
+        }
+      }
+      else if (winId == children [i])
+        winIdFound = true;
+
+      // According to tests, my own window ID is either on toplevel or two levels below.
+      // To avoid unneccessary recursion, we limit the search to two recursion levels,
+      // then to five recursion levels, and make a full recursive search only if that
+      // was unsuccessful.
+      else if (level > 1)
+        winIdFound = obscuredRegion (region, winId, ignoreId, children [i], level-1);
+      else if (level == -1)
+        if (! (winIdFound = obscuredRegion (region, winId, ignoreId, children [i], 0)))
+          if (! (winIdFound = obscuredRegion (region, winId, ignoreId, children [i], 1)))
+            winIdFound = obscuredRegion (region, winId, ignoreId, children [i], -1);
+    }
+
+    if (children != NULL)
+      XFree (children);
+  }
+
+  return winIdFound;
+}
+
+
+
+
 KMagZoomView::KMagZoomView(QWidget *parent, const char *name)
   : QScrollView(parent, name),
     m_selRect(0, 0, 128, 128, this),
@@ -83,6 +130,7 @@ KMagZoomView::KMagZoomView(QWidget *parent, const char *name)
     m_showMouse(1),
     m_zoom(1.0),
     m_rotation(0),
+    m_invert(false),
     m_fitToWindow(true)
 {
   KApplication::setGlobalMouseTracking(TRUE);
@@ -90,11 +138,6 @@ KMagZoomView::KMagZoomView(QWidget *parent, const char *name)
   viewport()->setBackgroundMode (NoBackground);
   viewport()->setFocusPolicy(QWidget::StrongFocus);
   
-  // init the zoom matrix
-  m_zoomMatrix.reset();
-  m_zoomMatrix.scale(m_zoom, m_zoom);
-  m_zoomMatrix.rotate(m_rotation);
-
   m_ctrlKeyPressed = false;
   m_shiftKeyPressed = false;
   m_refreshSwitch = true;
@@ -119,8 +162,7 @@ KMagZoomView::KMagZoomView(QWidget *parent, const char *name)
   // different ways to show the cursor.
   m_showMouseTypes << "Hidden" << "Box" << "Arrow" << "Actual";
 
-  if(m_fitToWindow)
-    fitToWindow();
+  updateMatrix();
 }
 
 KMagZoomView::~KMagZoomView()
@@ -191,44 +233,34 @@ void KMagZoomView::resizeEvent( QResizeEvent * e )
  */
 void KMagZoomView::drawContents ( QPainter * p, int clipx, int clipy, int clipw, int cliph )
 {
-  if(m_grabbedZoomedPixmap.isNull())
+  if(m_grabbedPixmap.isNull())
     return;
 
-  // Paint empty areas black
-  if (contentsX()+contentsWidth() < visibleWidth())
-    p->fillRect (
-        QRect (contentsX()+contentsWidth(), clipy, visibleWidth()-contentsX()-contentsWidth(), cliph)
-        & QRect (clipx, clipy, clipw, cliph),
-        Qt::black);
-  if (contentsY()+contentsHeight() < visibleHeight())
-    p->fillRect (
-        QRect (clipx, contentsY()+contentsHeight(), clipw, visibleHeight()-contentsY()-contentsHeight())
-        & QRect (clipx, clipy, clipw, cliph),
-        Qt::black);
-
   // A pixmap which will be eventually displayed
-  QPixmap *zoomView;
-
-  // Get mouse position relative to the image
-  QPoint mousePos = calcMousePos (m_refreshSwitch);
+  QRect areaToPaint = m_invertedMatrix.mapRect (QRect(clipx, clipy, clipw, cliph));
+  QPixmap clippedPixmap (areaToPaint.size());
+  clippedPixmap.fill (QColor (128, 128, 128));
+  areaToPaint &= QRect (QPoint (0,0), m_selRect.size());
+  bitBlt(&clippedPixmap, QPoint (0,0), &m_grabbedPixmap, areaToPaint);
 
   // show the pixel under mouse cursor
   if(m_showMouse) {
-
-    // Pixmap which will have the zoomed pixmap + mouse
-    zoomView = new QPixmap(m_grabbedZoomedPixmap);
-
     // paint the mouse cursor
-    paintMouseCursor(zoomView, mousePos);
-  } else { // do not show mouse
-    zoomView = &m_grabbedZoomedPixmap;
+    paintMouseCursor(&clippedPixmap, calcMousePos (m_refreshSwitch)-QPoint (areaToPaint.x(), areaToPaint.y()));
   }
 
-  // bitBlt this part on to the widget.
-  bitBlt(viewport(), QPoint (clipx-contentsX(), clipy-contentsY()), zoomView, QRect(clipx, clipy, clipw, cliph));
+  QPixmap zoomedPixmap;
+  zoomedPixmap = clippedPixmap.xForm (m_zoomMatrix);
 
-  if(zoomView != &m_grabbedZoomedPixmap)
-    delete zoomView;
+  if (m_invert) {
+    QImage zoomedImage;
+    zoomedImage = zoomedPixmap.convertToImage();
+    zoomedImage.invertPixels (false);
+    p->drawImage (QPoint (clipx-contentsX(), clipy-contentsY()), zoomedImage, zoomedImage.rect(),
+                  Qt::ThresholdDither | Qt::ThresholdAlphaDither | Qt::AvoidDither);
+  } else {
+    p->drawPixmap (QPoint (clipx, clipy), zoomedPixmap, zoomedPixmap.rect());
+  }
 }
 
 /**
@@ -240,11 +272,10 @@ void KMagZoomView::paintMouseCursor(QPaintDevice *dev, QPoint mousePos)
   if(!dev)
     return;
 
-  // painter for the zoom view
   QPainter pz(dev);
 
   if(m_latestCursorPos.x() >= 0 && m_latestCursorPos.x() < m_selRect.width() &&
-     m_latestCursorPos.y() >= 0 && m_latestCursorPos.y() < m_selRect.height() ) { // || updateMousePos) {
+     m_latestCursorPos.y() >= 0 && m_latestCursorPos.y() < m_selRect.height() ) {
     // mouse position is indeed inside the selRect
       
     // How to show the mouse :
@@ -254,7 +285,7 @@ void KMagZoomView::paintMouseCursor(QPaintDevice *dev, QPoint mousePos)
       // 1. Square around the pixel
       pz.setPen(Qt::white);
       pz.setRasterOp(Qt::XorROP);
-      pz.drawRect(mousePos.x()-1, mousePos.y()-1, (int)m_zoom+2, (int)m_zoom+2);
+      pz.drawRect(mousePos.x()-1, mousePos.y()-1, 3, 3);
       break;
 
     case 2:
@@ -266,17 +297,9 @@ void KMagZoomView::paintMouseCursor(QPaintDevice *dev, QPoint mousePos)
       QBitmap sCursor( 16, 16, left_ptr_bits, TRUE );
       QBitmap mask( 16, 16, left_ptrmsk_bits, TRUE );
       sCursor.setMask(mask);
-      sCursor = sCursor.xForm(m_zoomMatrix);
 
       // since hot spot is at 3,1
-      if (m_rotation == 0)
-        pz.drawPixmap(mousePos.x()-(int)(3.0*m_zoom), mousePos.y()-(int)m_zoom, sCursor);
-      else if (m_rotation == 90)
-        pz.drawPixmap(mousePos.x()-(int)(16.0*m_zoom), mousePos.y()-(int)(3.0*m_zoom), sCursor);
-      else if (m_rotation == 180)
-        pz.drawPixmap(mousePos.x()-(int)(13.0*m_zoom), mousePos.y()-(int)(16.0*m_zoom), sCursor);
-      else if (m_rotation == 270)
-        pz.drawPixmap(mousePos.x()-(int)m_zoom, mousePos.y()-(int)(13.0*m_zoom), sCursor);
+      pz.drawPixmap(mousePos.x()-3, mousePos.y()-1, sCursor);
     }
     break;
 
@@ -298,10 +321,9 @@ void KMagZoomView::paintMouseCursor(QPaintDevice *dev, QPoint mousePos)
           QBitmap sCursor( 16, 16, left_ptr_bits, TRUE );
           QBitmap mask( 16, 16, left_ptrmsk_bits, TRUE );
           sCursor.setMask(mask);
-          sCursor = sCursor.xForm(m_zoomMatrix);
 
           // since hot spot is at 3,1
-          pz.drawPixmap(mousePos.x()-(int)(3.0*m_zoom), mousePos.y()-(int)m_zoom, sCursor);
+          pz.drawPixmap(mousePos.x()-3, mousePos.y()-1, sCursor);
         }
         break;
         default:
@@ -333,19 +355,8 @@ QPoint KMagZoomView::calcMousePos(bool updateMousePos)
     m_latestCursorPos -= QPoint(m_selRect.x(), m_selRect.y());
   }
 
-  // get coordinates of the pixel w.r.t. the zoomed pixmap
-  if (m_rotation == 90)
-    return QPoint ((int)((float)(m_selRect.height()-m_latestCursorPos.y())*m_zoom),
-                   (int)((float)m_latestCursorPos.x()*m_zoom));
-  else if (m_rotation == 180)
-    return QPoint ((int)((float)(m_selRect.width()-m_latestCursorPos.x())*m_zoom),
-                   (int)((float)(m_selRect.height()-m_latestCursorPos.y())*m_zoom));
-  else if (m_rotation == 270)
-    return QPoint ((int)((float)m_latestCursorPos.y()*m_zoom),
-                   (int)((float)(m_selRect.width()-m_latestCursorPos.x())*m_zoom));
-  else
-    return QPoint ((int)((float)m_latestCursorPos.x()*m_zoom),
-                   (int)((float)m_latestCursorPos.y()*m_zoom));
+  // get coordinates of the pixel w.r.t. the pixmap
+  return QPoint (m_latestCursorPos.x(), m_latestCursorPos.y());
 }
 
 
@@ -545,66 +556,25 @@ void KMagZoomView::mouseMoveEvent(QMouseEvent *e)
     m_selRect.setRight(e->globalX());
     m_selRect.setBottom(e->globalY());
     m_selRect.update();
+
     grabFrame();
   } else if(m_mouseMode == MoveSelection) {
-     QPoint newCenter;
+    QPoint newCenter;
 
     // set new center to be the current mouse position
-    newCenter = e->globalPos();
-
-    // make sure the mouse position is not taking the grab window outside
-    // the display
-    if(newCenter.x() < m_selRect.width()/2) {
-      // set X to the minimum possible X
-      newCenter.setX(m_selRect.width()/2);
-    } else if(newCenter.x() >=  QApplication::desktop()->width()-m_selRect.width()/2) {
-      // set X to the maximum possible X
-      newCenter.setX(QApplication::desktop()->width()-m_selRect.width()/2-1);
-    }
-
-    if(newCenter.y() < m_selRect.height()/2) {
-      // set Y to the minimum possible Y
-      newCenter.setY(m_selRect.height()/2);
-    } else if(newCenter.y() >=  QApplication::desktop()->height()-m_selRect.height()/2) {
-      // set Y to the maximum possible Y
-      newCenter.setY(QApplication::desktop()->height()-m_selRect.height()/2-1);
-    }
-    // move to the new center
-    m_selRect.moveCenter(newCenter);
-    // update the grab rectangle display
+    m_selRect.moveCenter(e->globalPos());
     m_selRect.update();
+
     grabFrame();
   } else if(m_mouseMode == GrabSelection) {
-     QPoint newPos;
+    QPoint newPos;
 
     // get new position
     newPos = e->globalPos();
-
     QPoint delta = (newPos - m_oldMousePos)/m_zoom;
-    QPoint newCenter = m_oldCenter-delta;
-
-    // make sure the mouse position is not taking the grab window outside
-    // the display
-    if(newCenter.x() < m_selRect.width()/2) {
-      // set X to the minimum possible X
-      newCenter.setX(m_selRect.width()/2);
-    } else if(newCenter.x() >=  QApplication::desktop()->width()-m_selRect.width()/2) {
-      // set X to the maximum possible X
-      newCenter.setX(QApplication::desktop()->width()-m_selRect.width()/2-1);
-    }
-
-    if(newCenter.y() < m_selRect.height()/2) {
-      // set Y to the minimum possible Y
-      newCenter.setY(m_selRect.height()/2);
-    } else if(newCenter.y() >=  QApplication::desktop()->height()-m_selRect.height()/2) {
-      // set Y to the maximum possible Y
-      newCenter.setY(QApplication::desktop()->height()-m_selRect.height()/2-1);
-    }
-            
-    // move to the new center  
-    m_selRect.moveCenter(newCenter);
-    // update the grab rectangle display
+    m_selRect.moveCenter(m_oldCenter-delta);
     m_selRect.update();
+
     grabFrame();
   }
 }
@@ -648,12 +618,7 @@ void KMagZoomView::keyPressEvent(QKeyEvent *e)
   else if (e->key() == QKeyEvent::Key_Right)
   {
     if (e->state() & QKeyEvent::ControlButton)
-    {
-      if (m_selRect.right()+offset >= QApplication::desktop()->width())
-        m_selRect.setRight (QApplication::desktop()->width()-1);
-      else
-        m_selRect.setRight (m_selRect.right()+offset);
-    }
+      m_selRect.setRight (m_selRect.right()+offset);
     else if (contentsX() < contentsWidth()-visibleWidth())
     {
       offset = (int)(offset*m_zoom);
@@ -663,12 +628,8 @@ void KMagZoomView::keyPressEvent(QKeyEvent *e)
         setContentsPos (contentsWidth()-visibleWidth(), contentsY());
     }
     else if (m_followMouse == false)
-    {
-      if (m_selRect.right()+offset >= QApplication::desktop()->width())
-        m_selRect.moveTopRight (QPoint (QApplication::desktop()->width()-1, m_selRect.top()));
-      else
-        m_selRect.moveBy (offset,0);
-    }
+      m_selRect.moveBy (offset,0);
+
     m_selRect.update();
   }
   else if (e->key() == QKeyEvent::Key_Up)
@@ -700,12 +661,7 @@ void KMagZoomView::keyPressEvent(QKeyEvent *e)
   else if (e->key() == QKeyEvent::Key_Down)
   {
     if (e->state() & QKeyEvent::ControlButton)
-    {
-      if (m_selRect.bottom()+offset >= QApplication::desktop()->height())
-        m_selRect.setBottom (QApplication::desktop()->height()-1);
-      else
-        m_selRect.setBottom (m_selRect.bottom()+offset);
-    }
+      m_selRect.setBottom (m_selRect.bottom()+offset);
     else if (contentsY() < contentsHeight()-visibleHeight())
     {
       offset = (int)(offset*m_zoom);
@@ -715,12 +671,7 @@ void KMagZoomView::keyPressEvent(QKeyEvent *e)
         setContentsPos (contentsX(), contentsHeight()-visibleHeight());
     }
     else if (m_followMouse == false)
-    {
-      if (m_selRect.bottom()+offset >= QApplication::desktop()->height())
-        m_selRect.moveBottomLeft (QPoint (m_selRect.left(), QApplication::desktop()->height()-1));
-      else
-        m_selRect.moveBy (0, offset);
-    }
+      m_selRect.moveBy (0, offset);
     m_selRect.update();
   }
   else
@@ -737,6 +688,12 @@ void KMagZoomView::keyReleaseEvent(QKeyEvent *e)
     e->ignore();
 }
 
+void KMagZoomView::contextMenuEvent (QContextMenuEvent *e)
+{
+ emit contextMenu(e->globalPos());
+ e->accept();
+}
+
 void KMagZoomView::focusOutEvent(QFocusEvent *e)
 {
   if(e->lostFocus() == TRUE) {
@@ -753,44 +710,12 @@ void KMagZoomView::focusOutEvent(QFocusEvent *e)
  */
 void KMagZoomView::fitToWindow()
 {
-  unsigned int newWidth, newHeight;
-  
-  // this is a temporary solution, cast, maybe newWidth and newHeight should be float
-  if ((m_rotation == 90) || (m_rotation == 270))
-  {
-    newWidth = static_cast<unsigned int>(visibleHeight()/m_zoom);
-    newHeight = static_cast<unsigned int>(visibleWidth()/m_zoom);
-  } else {
-    newWidth = static_cast<unsigned int>(visibleWidth()/m_zoom);
-    newHeight = static_cast<unsigned int>(visibleHeight()/m_zoom);
-  }
-
   QPoint currCenter = m_selRect.center();
-
-  m_selRect.setWidth(newWidth);
-  m_selRect.setHeight(newHeight);
-
-   // make sure the selection window does not go outside of the display
-   if(currCenter.x() < m_selRect.width()/2) {
-     // set X to the minimum possible X
-     currCenter.setX(m_selRect.width()/2);
-   } else if(currCenter.x() >=  QApplication::desktop()->width()-m_selRect.width()/2) {
-     // set X to the maximum possible X
-     currCenter.setX(QApplication::desktop()->width()-m_selRect.width()/2-1);
-   }
-
-   if(currCenter.y() < m_selRect.height()/2) {
-     // set Y to the minimum possible Y
-     currCenter.setY(m_selRect.height()/2);
-   } else if(currCenter.y() >=  QApplication::desktop()->height()-m_selRect.height()/2) {
-     // set Y to the maximum possible Y
-     currCenter.setY(QApplication::desktop()->height()-m_selRect.height()/2-1);
-   }
-
+  QRect newRect = m_invertedMatrix.mapRect (QRect(0, 0, visibleWidth(), visibleHeight()));
+  m_selRect.setSize (newRect.size());
   m_selRect.moveCenter(currCenter);
-  // update the grab rectangle display
   m_selRect.update();
-//  m_fitToWindow = true;
+
   viewport()->repaint(false);
 }
 
@@ -817,29 +742,7 @@ void KMagZoomView::grabFrame()
      QPoint newCenter;
 
     // set new center to be the current mouse position
-    newCenter = QCursor::pos();
-
-    // make sure the mouse position is not taking the grab window outside
-    // the display
-    if(newCenter.x() < m_selRect.width()/2) {
-      // set X to the minimum possible X
-      newCenter.setX(m_selRect.width()/2);
-    } else if(newCenter.x() >=  QApplication::desktop()->width()-m_selRect.width()/2) {
-      // set X to the maximum possible X
-      newCenter.setX(QApplication::desktop()->width()-m_selRect.width()/2-1);
-    }
-
-    if(newCenter.y() < m_selRect.height()/2) {
-      // set Y to the minimum possible Y
-      newCenter.setY(m_selRect.height()/2);
-    } else if(newCenter.y() >=  QApplication::desktop()->height()-m_selRect.height()/2) {
-      // set Y to the maximum possible Y
-      newCenter.setY(QApplication::desktop()->height()-m_selRect.height()/2-1);
-    }
-    // move to the new center
-    m_selRect.moveCenter(newCenter);
-
-    // update the grab rectangle display
+    m_selRect.moveCenter(QCursor::pos());
     m_selRect.update();
   }
 
@@ -852,11 +755,23 @@ void KMagZoomView::grabFrame()
   m_grabbedPixmap = QPixmap::grabWindow(QApplication::desktop()->winId(), selRect.x(), selRect.y(),
                                         selRect.width(), selRect.height());
 
-  // zoom the image
-  m_grabbedZoomedPixmap = m_grabbedPixmap.xForm(m_zoomMatrix);
+  // If the KMag window itself is in the screenshot, then it need to be filled with gray to avoid recursion
+  QPoint globalPos = viewport()->mapToGlobal (viewport()->rect().topLeft());
+  QRegion intersection (globalPos.x(), globalPos.y(), viewport()->width(), viewport()->height(), QRegion::Rectangle);
+  intersection &= QRegion (selRect, QRegion::Rectangle);
+
+  // We don't want to overpaint other windows that happen to be on top
+  obscuredRegion (intersection, topLevelWidget()->winId(), m_selRect.winId());
+  intersection.translate (-selRect.x(), -selRect.y());
+
+  QPainter painter (&m_grabbedPixmap, true);
+  QMemArray<QRect> rects (intersection.rects());
+  for (uint i = 0; i < rects.size(); i++)
+    painter.fillRect (rects[i], QBrush (QColor (128, 128, 128)));
 
   // call repaint to display the newly grabbed image
-  resizeContents (m_grabbedZoomedPixmap.width(), m_grabbedZoomedPixmap.height());
+  QRect newSize = m_zoomMatrix.mapRect (m_grabbedPixmap.rect());
+  resizeContents (newSize.width(), newSize.height());
   viewport()->repaint(false);
 }
 
@@ -894,20 +809,8 @@ void KMagZoomView::toggleRefresh()
  */
 void KMagZoomView::setZoom(float zoom)
 {
-  // use this zoom
   m_zoom = zoom;
-
-  // update selection window size when zooming in if necessary
-  if (m_fitToWindow)
-    fitToWindow();
-
-  // recompute the zoom matrix
-  m_zoomMatrix.reset();
-  m_zoomMatrix.scale(m_zoom, m_zoom);
-  m_zoomMatrix.rotate(m_rotation);
-
-  m_grabbedZoomedPixmap = m_grabbedPixmap.xForm(m_zoomMatrix);
-
+  updateMatrix();
   viewport()->repaint();
 }
 
@@ -916,20 +819,17 @@ void KMagZoomView::setZoom(float zoom)
  */
 void KMagZoomView::setRotation(int rotation)
 {
-  // use this rotation
   m_rotation = rotation;
+  updateMatrix();
+  viewport()->repaint();
+}
 
-  // update selection window size if necessary
-  if (m_fitToWindow)
-    fitToWindow();
-
-  // recompute the zoom matrix
-  m_zoomMatrix.reset();
-  m_zoomMatrix.scale(m_zoom, m_zoom);
-  m_zoomMatrix.rotate(m_rotation);
-
-  m_grabbedZoomedPixmap = m_grabbedPixmap.xForm(m_zoomMatrix);
-
+/**
+ * This function sets whether the magnified image is shown inverted
+ */
+void KMagZoomView::setInvertation(bool invert)
+{
+  m_invert = invert;
   viewport()->repaint();
 }
 
@@ -994,15 +894,34 @@ QStringList KMagZoomView::getShowMouseStringList() const
 QPixmap KMagZoomView::getPixmap()
 {
   // show the pixel under mouse cursor
-  if(m_showMouse && !m_grabbedZoomedPixmap.isNull()) {
-    // Pixmap which will have the zoomed pixmap + mouse
-    QPixmap zoomView(m_grabbedZoomedPixmap);
+  if(m_showMouse && !m_grabbedPixmap.isNull()) {
+    // Pixmap which will have the pixmap + mouse
+    QPixmap mousePixmap(m_grabbedPixmap);
 
     // paint the mouse cursor w/o updating to a newer position
-    paintMouseCursor(&zoomView, calcMousePos(false));
+    paintMouseCursor(&mousePixmap, calcMousePos(false));
     
-    return(zoomView);
+    return(mousePixmap);
   } else { // no mouse cursor
-     return(m_grabbedZoomedPixmap);
+     return(m_grabbedPixmap);
   }
 }
+
+/**
+ * Update the magnification matrix
+ */
+void KMagZoomView::updateMatrix()
+{
+  // update selection window size if necessary
+  if (m_fitToWindow)
+    fitToWindow();
+
+  // recompute the zoom matrix
+  m_zoomMatrix.reset();
+  m_zoomMatrix.scale(m_zoom, m_zoom);
+  m_zoomMatrix.rotate(m_rotation);
+
+  bool inverted;
+  m_invertedMatrix = m_zoomMatrix.invert (&inverted);
+
+}
